#!/bin/sh

PARSER_DIR=/opt/novell/nam/autoscale/scripts
NAM_DIR=/tmp/novell_access_manager/autoscale
LOG_OUTPUT="$NAM_DIR/idp_instance.log"
LOG_ERROR=$LOG_OUTPUT

FAILURE_FILE=$NAM_DIR/../failure
JCC_ID_FILE=/opt/novell/nids/lib/webapp/WEB-INF/jccid.dat
EC2_INSTANCE_REGION=""

get_ec2_lifecycle_details()
{
    #get instance id from instance metadata
    EC2_INSTANCE_ID=$( curl -s http://169.254.169.254/latest/meta-data/instance-id  )
    # get and parse instance region from instance metadata
    EC2_INSTANCE_REGION=$( curl -s http://169.254.169.254/latest/meta-data/placement/availability-zone  )
    EC2_INSTANCE_REGION=$( echo "$EC2_INSTANCE_REGION" | sed 's|[a-z]$||g' )

    # get the atuoscale group name
    AUTOSCALE_GROUPNAME=$( aws autoscaling describe-auto-scaling-instances --instance-ids $EC2_INSTANCE_ID --region $EC2_INSTANCE_REGION --query  AutoScalingInstances[0].AutoScalingGroupName --output text )

    #Get the life cylce launch hook name
    AUTOSCALE_LIFECYCLE_TERMINATE_HOOK=$( aws autoscaling describe-lifecycle-hooks --auto-scaling-group-name $AUTOSCALE_GROUPNAME --region  $EC2_INSTANCE_REGION --query LifecycleHooks[1].LifecycleHookName --output text )

    #Get lifecycle hook heartbeat timeout
    LIFECYCLE_HEARTBEAT_TIMEOUT=$( aws autoscaling describe-lifecycle-hooks --auto-scaling-group-name  $AUTOSCALE_GROUPNAME  --region  $EC2_INSTANCE_REGION --query LifecycleHooks[1].HeartbeatTimeout --output text )

    export AUTOSCALE_GROUPNAME
    export AUTOSCALE_LIFECYCLE_TERMINATE_HOOK
    export LIFECYCLE_HEARTBEAT_TIMEOUT
}


init_region()
{
	# get and parse instance region from instance metadata
	EC2_INSTANCE_REGION=$( curl -s http://169.254.169.254/latest/meta-data/placement/availability-zone )
	EC2_INSTANCE_REGION=$( echo "$EC2_INSTANCE_REGION" | sed 's|[a-z]$||g' )

	echo "region set to: $EC2_INSTANCE_REGION" >>$LOG_OUTPUT
}
get_secret_value()
{
	local secret_name=$1

	echo "reading the value for: $secret_name" >>$LOG_OUTPUT

	# read the values from secrets manager
	#echo `aws secretsmanager get-secret-value --secret-id "$secret_name" --region ap-south-1 --query SecretString --output text`

	# read the values from EC2 parameter store
	local value=`aws ssm get-parameter --name "$secret_name" --with-decryption --query Parameter.Value --output text --region "$EC2_INSTANCE_REGION"` 1>>$LOG_OUTPUT 2>>$LOG_ERROR
	echo $value
}

getDeviceID()
{
    local deviceID="idp-"$(cat $JCC_ID_FILE | awk '{print $1}')
    echo $deviceID
}

getBasicDetails()
{
	init_region

	ADMIN_NAME=$( get_secret_value AdminUsername )
	ADMIN_PASS=$( get_secret_value AdminPassword )
	AC_IP=$( get_secret_value AdminConsolePrivateIP )
	IDP_CLUSTER_NAME=$( get_secret_value IDPClusterName )
	IDP_CLUSTER_ID=$( get_secret_value IDPClusterID )
	AWS_S3BUCKET=$( get_secret_value S3BucketName )
	AWS_S3REGION=$( get_secret_value S3BucketRegion )
	IDP_DEVICE_ID=$(getDeviceID)

    if [[ -z "$ADMIN_NAME" || -z "$ADMIN_PASS" || -z "$IDP_CLUSTER_NAME" || -z "$IDP_CLUSTER_ID" || -z "$IDP_DEVICE_ID" ]]; then
        echo "Error fetching required details to perform scale down of IDP. Please delete IDP manually from Admin Console" >> $LOG_ERROR
        exit 1
    fi

	export ADMIN_NAME
	export ADMIN_PASS
	export AC_IP
	export IDP_CLUSTER_NAME
	export IDP_CLUSTER_ID
	export IDP_DEVICE_ID
	export $AWS_S3BUCKET
    export $AWS_S3REGION
}

copyInstanceLogsToS3()
{
    echo "Backing up application log files to AWS S3 store" >> "$LOG_OUTPUT"
    log_folder=/tmp/novell_access_manager/autoscale
    instanceIP=`curl http://169.254.169.254/latest/meta-data/local-ipv4`
    tar_bundle=idp_"$EC2_INSTANCE_ID"_"$instanceIP".tar.gz

    catalina_log=/opt/novell/nam/idp/logs/catalina.out
    jcc_logs=/opt/novell/devman/jcc/logs/jcc-0.log.0
    jcc_configure_logs=jcc_logs=/opt/novell/devman/jcc/logs/configure.log.0

    tar -czvf $tar_bundle $log_folder/* $catalina_log $jcc_logs $jcc_configure_logs

    aws s3 cp $tar_bundle s3://$AWS_S3BUCKET/logs/ --region $AWS_S3REGION

    rm -f $tar_bundle
}

# At any case, this script should exit abandoning the instance whatever may be the state - To avoid unnecessary usage of resources in AWS
ec2_instance_abandon ()
{
    copyInstanceLogsToS3
    echo "Triggering AWS instance abandon for instance $EC2_INSTANCE_ID in the region $EC2_INSTANCE_REGION" >> "$LOG_OUTPUT"
    #Abandon the instance
    aws autoscaling complete-lifecycle-action --lifecycle-action-result ABANDON  --instance-id $EC2_INSTANCE_ID  --lifecycle-hook-name $AUTOSCALE_LIFECYCLE_TERMINATE_HOOK --auto-scaling-group-name $AUTOSCALE_GROUPNAME --region $EC2_INSTANCE_REGION
}
trap ec2_instance_abandon EXIT


ec2_instance_continue ()
{
   #Continue the instance
   aws autoscaling complete-lifecycle-action --lifecycle-action-result CONTINUE  --instance-id $EC2_INSTANCE_ID  --lifecycle-hook-name  $AUTOSCALE_LIFECYCLE_TERMINATE_HOOK --auto-scaling-group-name $AUTOSCALE_GROUPNAME --region $EC2_INSTANCE_REGION
}

deleteOrphanIDP(){

    echo "Removing Improper Node : Deleting IDP which could not be properly assigned to cluster"

    local response_code=$(curl --write-out %{http_code} --silent --output curl_response -k -X DELETE "https://$AC_IP:8443/amsvc/v1/devices/idp/$IDP_DEVICE_ID" -u cn=$ADMIN_NAME,o=novell:$ADMIN_PASS)

    echo -e "Received response : \n""Status : $response_code \n""Message : $(<curl_response)" >> "$LOG_OUTPUT"
    rm -f curl_response

    if [[ ("$response_code" != 200) ]]; then
        echo "Failed deleting the IDP server. Please delete it manually from Admin Console" >> "$LOG_ERROR"
        exit 3
    else
        echo "Successfully deleted improper IDP server from the Admin Console" >> "$LOG_OUTPUT"
    fi
}

deleteIDPFromCluster()
{
    echo "Scaling down : Deleting IDP From the Cluster" >> "$LOG_OUTPUT"

    local response_code=$(curl --write-out %{http_code} --silent --output curl_response -k -X DELETE "https://$AC_IP:8443/amsvc/v1/idpclusters/$IDP_CLUSTER_ID/devices/$IDP_DEVICE_ID" -u cn=$ADMIN_NAME,o=novell:$ADMIN_PASS)

    echo -e "Received response : \n""Status : $response_code \n""Message : $(<curl_response)" >> $LOG_OUTPUT
    rm -f curl_response

    if [[ ("$response_code" != 200) ]]; then
        echo "Failed deleting the IDP server. Please delete it manually from Admin Console" >> "$LOG_ERROR"
        exit 2
    fi

    echo "Successfully deleted IDP server from the Cluster" >> "$LOG_OUTPUT"
}


checkForUserSession()
{
    response_code=$(curl --write-out %{http_code} --silent --output curl_response -k -X GET "https://$AC_IP:8443/roma/rest/$IDP_CLUSTER_ID/sessions/devices/$IDP_DEVICE_ID" -u cn=$ADMIN_NAME,o=novell:$ADMIN_PASS)
    if [[ ("$response_code" == 200) ]]; then
        json_body=$(<curl_response)
        parser_response=$(python -c 'execfile('"\"$PARSER_DIR/json_parser.py"\"'); get_sessions_in_idp()' "$json_body" "$LOG_ERROR" 2>>$LOG_ERROR )
        if [[ ("$parser_response" != "failure") ]]; then
            numberOfUserSession=$parser_response
        fi
    fi
    rm -f curl_response
    if [[ ("$numberOfUserSession" == 0) ]]; then
        echo "This server does not have any stored session. Hence proceeding with delete" >> "$LOG_ERROR"
        echo "false"
    elif [[ ("$numberOfUserSession" -ge 1) ]]; then
        echo "Found user sessions - So waiting till session gets terminated" >> "$LOG_ERROR"
        echo "true"
    else
        echo "Error while parsing. Check the device details provided" >> "$LOG_ERROR"
        echo "false"
    fi
}

updateCluster(){

    echo "Updating the Cluster : "$IDP_CLUSTER_NAME >> "$LOG_OUTPUT"
    local body='{"update":"all"}'
    curl -k -X PUT "https://$AC_IP:8443/amsvc/v1/idpclusters/$IDP_CLUSTER_ID" -H "Content-Type: application/json" -d $body -u cn=$ADMIN_NAME,o=novell:$ADMIN_PASS
}

checkSessionAndDelete(){
    hasUserSession=$(checkForUserSession)
    if [[ ("$hasUserSession" == "false") ]]; then
        deleteIDPFromCluster
        isServerDeleted="true"
        updateCluster
    fi
}

#################
#      MAIN     #
#################

get_ec2_lifecycle_details
getBasicDetails

#if node_assigned file is present, then something went wrong with the Installation. Value inside will say at what state failure happened
# "stage1" - failure during import
# "stage2" - failure after import but before node assignment
# "stage3" - failure after node assigned to cluster

failure_stage=""
[ -f $FAILURE_FILE ] && failure_stage=$(cat $FAILURE_FILE)

if [ "$failure_stage" == "stage1" ]; then
    exit 0
elif [ "$failure_stage" == "stage2" ]; then
    deleteOrphanIDP
    exit 0
elif [ "$failure_stage" == "stage3" ]; then
    deleteIDPFromCluster
    updateCluster
    exit 0
fi

isServerDeleted="false"
# Safe window time should be there to do all cleaning and backup operations before termination and also to compensate delay in between operations (Setting it now to be 3 minutes).
# This is done because "LIFECYCLE_HEARTBEAT_TIMEOUT" is a hard limit where instance will be abandoned irrespective of the state.
SAFE_WINDOW_TIME=180
TIME_LIMIT_FOR_SERVER_EXIT="($LIFECYCLE_HEARTBEAT_TIMEOUT)"- $SAFE_WINDOW_TIME
SECONDS=0
#checkSessionAndDelete
while [ "$isServerDeleted" == "false" ]  ; do
   checkSessionAndDelete
   "$isServerDeleted" == "true" && break
   sleep 60
   if [[ $SECONDS -gt $TIME_LIMIT_FOR_SERVER_EXIT ]]; then
       echo "Maximum wait time limit set for the IDP deletion reached. Hence, Forcing IDP deletion of the server" >> "$LOG_OUTPUT"
       deleteIDPFromCluster
       isServerDeleted="true"
       updateCluster
   fi
done
